// C Source File
// Created 11/21/2010; 2:30:54 PM

//******************************************************************************
//
// Project:         Jumpman 89
//
// file name:       jm_levels.c
//
// author:          David Randall "Ranman" 
//                  derandall99@gmail.com
//
// notes::          Jumpman was originally created by Randy Glover for the
//                  Atari 400, Commodore 64, IBM PCjr, and Apple II in 1982
//
//******************************************************************************


#include <tigcclib.h>
#include "extgraph.h"

#include "BasicTypes.h"
#include "GrayUtils.h"
#include "FileUtils.h"
#include "FontUtils.h"
#include "TimeUtils.h"
#include "BasicUtils.h"
#include "KeyUtils.h"

#include "jm_levels.h"
#include "jm_bullets.h"
#include "jm_scores.h"
#include "jm_collectables.h"
#include "jm_platforms.h"


//#define USE_INTERNAL_LEVEL_PACK

LEVEL_Type internal_level_pack[] =
{
#ifdef USE_INTERNAL_LEVEL_PACK
	{
		"TESTER",
		{
			//{ 1, GAME_PIECE_Jumpman   , FLAG_VA,  19, 52,  SPECIAL_ABILITY_Spear},
			{ 1, GAME_PIECE_Jumpman   , FLAG_VA,  19, 52,  SPECIAL_ABILITY_None},

			{ 1, GAME_PIECE_Girder_Hor, FLAG_VA,  0, 88, 40},
			//{ 3, GAME_PIECE_Ladder, FLAG_VA, 19,  4, 21},

			{ 1, GAME_PIECE_Girder_Hor, FLAG_VA,  0, 60, 40},
			{ 1, GAME_PIECE_Girder_Hor, FLAG_VA,  0, 24, 10},
			
			{ 1, GAME_PIECE_Girder_Hor, FLAG_VA,  15, 42, 10},

			{ 3, GAME_PIECE_Ladder, FLAG_VA,  1, 20, 17},
			{ 3, GAME_PIECE_Ladder, FLAG_VA,  6, 53, 9},
			{ 3, GAME_PIECE_Ladder, FLAG_VA,  11, 54, 9},
			{ 3, GAME_PIECE_Ladder, FLAG_VA,  16, 55, 9},
			//{ 3, GAME_PIECE_Ladder, FLAG_VA,  26-1, 56, 9},
			//{ 3, GAME_PIECE_Ladder, FLAG_VA,  32-1, 57, 9},
			//{ 3, GAME_PIECE_Ladder, FLAG_VA,  38-1, 58, 9},

			{14, GAME_PIECE_Rope_Down, FLAG_VA, 21, 57,  6},
			{13, GAME_PIECE_Rope_Up, FLAG_VA, 26, 63,  4},
			{13, GAME_PIECE_Rope_Up, FLAG_VA, 34, 61,  5},
			{14, GAME_PIECE_Rope_Down, FLAG_VA, 31, 56,  6},
			{13, GAME_PIECE_Rope_Up, FLAG_VA, 36, 63,  5},

			{21, GAME_PIECE_Spear, FLAG_VA, 0, 0, BULLET_ACTION_None},
			{21, GAME_PIECE_Spear, FLAG_VA, 0, 0, BULLET_ACTION_None},
			{21, GAME_PIECE_Spear, FLAG_VA, 0, 0, BULLET_ACTION_None},
			{21, GAME_PIECE_Spear, FLAG_VA, 0, 0, BULLET_ACTION_None},
			{21, GAME_PIECE_Spear, FLAG_VA, 0, 0, BULLET_ACTION_None},
			{21, GAME_PIECE_Spear, FLAG_VA, 0, 0, BULLET_ACTION_None},
			{21, GAME_PIECE_Spear, FLAG_VA, 0, 0, BULLET_ACTION_None},
			{21, GAME_PIECE_Spear, FLAG_VA, 0, 0, BULLET_ACTION_None},
			
			{17, GAME_PIECE_Bomb, FLAG_VA, 6, 3,  0},

/*
			{ 2, 1, FLAG_VA,  0,  8,  8},
			{ 3, 7, FLAG_VA,  1,  4, 21},
			{ 4, 1, FLAG_VA, 32,  8,  8},
			{ 5, 7, FLAG_VA, 37,  4, 21},
			{ 6, 1, FLAG_VA, 16,  8,  8},
			{ 7, 7, FLAG_VA, 19,  4,  7},
			{ 8, 1, FLAG_VA, 15, 28, 10},
			{ 9, 7, FLAG_VA, 16, 24,  6},
			{10, 7, FLAG_VA, 22, 24,  6},
			{11, 1, FLAG_VA, 14, 48,  5},
			{12, 1, FLAG_VA, 21, 48,  5},
			{13, 5, FLAG_VA, 14, 48,  5},
			{14, 6, FLAG_VA, 24, 44,  8},
			{15, 1, FLAG_VA, 13, 68, 14},
			{16, 7, FLAG_VA, 19, 64,  6},
*/

//			{17, GAME_PIECE_Bomb, FLAG_VA, 6, 3,  0},
//			{18, GAME_PIECE_Bomb, FLAG_VA, 16, 3,  0},
//			{19, GAME_PIECE_Bomb, FLAG_VA, 22, 3,  0},
//			{20, GAME_PIECE_Bomb, FLAG_VA, 32, 3,  0},
//			{21, GAME_PIECE_Hailstone, FLAG_VA, 0, 0, BULLET_ACTION_None},
//			{22, GAME_PIECE_Hailstone, FLAG_VA, 0, 0, BULLET_ACTION_None},
//			{23, GAME_PIECE_Hailstone, FLAG_VA, 0, 0, BULLET_ACTION_None},
//			{24, GAME_PIECE_Hailstone, FLAG_VA, 0, 0, BULLET_ACTION_None},
		//	{22, GAME_PIECE_Jumping_Block, FLAG_VA, 0, 0, BULLET_ACTION_HorzAndVert},
		//	{23, GAME_PIECE_Zig_Zag_Bullet, FLAG_VA, 0, 0, BULLET_ACTION_HorzAndVert},
		//	{24, GAME_PIECE_Hailstone, FLAG_VA, 0, 0, BULLET_ACTION_None},
		//	{25, GAME_PIECE_Hailstone, FLAG_VA, 0, 0, BULLET_ACTION_None},
		//	{26, GAME_PIECE_Hailstone, FLAG_VA, 0, 0, BULLET_ACTION_None},
		//	{25, GAME_PIECE_Jumping_Block, FLAG_VA, 0, 0, BULLET_ACTION_HorzAndVert},
		//	{26, GAME_PIECE_Jumping_Block, FLAG_VA, 0, 0, BULLET_ACTION_HorzAndVert},
		//	{27, GAME_PIECE_Jumping_Block, FLAG_VA, 0, 0, BULLET_ACTION_HorzAndVert},
		//	{28, GAME_PIECE_Jumping_Block, FLAG_VA, 0, 0, BULLET_ACTION_HorzAndVert},
//			{99, GAME_PIECE_Bomb, FLAG_VA, 10,84,  0},
			{ 0, 0, 0, 0, 0, 0},
		},
		1500,1,0,
		0,0,0,0,
		false,
		false,
	},
	{
		"APOGEE FONT TEST",
		{
			{  1, GAME_PIECE_Jumpman,     FLAG_VA,  19, 80,  SPECIAL_ABILITY_None},

			{ 10, GAME_PIECE_Girder_Hor,  FLAG_VA,   0, 88,  40},

			{ 60, GAME_PIECE_Bomb,        FLAG_VAS,  8, 84, 100},
			{ 60, GAME_PIECE_Bomb,        FLAG_VA, 38, 84, 100},

			{100, GAME_PIECE_Script, FLAG_BONUS, FLAG_NONE, 50, 0},

			{ 0, 0, 0, 0, 0, 0},
		},
		1500,2,0,
		0,0,0,0,
		false,
		true,
	},
#endif
};



//B  4, 60 ->  4, 80 ->  17, 40 ->   4, 14 ->   4, 60 ->  4, 80 ->   4, 60
//G 29, 80 -> 31, 60 ->  34, 14 ->  16,  0 ->  12, 40 ->  4, 60 ->  29, 80

//	DIR_None
//	DIR_Up,
//	DIR_Down,
//	DIR_Left,
//	DIR_Right,

typedef struct
{
	char name[32];
	char file_name[10];
	uint4 compression_address;
} LEVEL_PACK_Name_Type;

LEVEL_PACK_Name_Type level_pack_list[10];
LEVEL_Type* external_level;
LEVEL_Type current_level;
int2 level_number;
bool level_complete;
uint1 level_toggle_bits;
//uint1 external_level_buffer[30*1024 + 256];
uint1* external_level_buffer;



void DisplayNextLevel(void)
{
#if 0
	char line_14[] = "~~~~~~~~~~~~~~";
	char line_26[] = "~~~~~~~~~~~~~~~~~~~~~~~~~~";
	char line_40[] = "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~";

	GrayUserDrawStr(0,34,    line_14, COLOR_LGray, 0x00);
	GrayUserDrawStr(4*14+1,34, " NEXT LEVEL ", COLOR_White, 0x00);
	GrayUserDrawStr(4*26,34, line_14, COLOR_LGray, 0x00);

	GrayUserDrawStr(0,46, line_40, COLOR_DGray, 0x00);
	GrayUserDrawStr(0,52, line_40, COLOR_DGray, 0x00);
	GrayUserDrawStr(0,58, line_40, COLOR_DGray, 0x00);
	GrayUserDrawStr(4*7,52, line_26, COLOR_White, 0x00);
	GrayUserDrawStr(80 - (strlen(current_level.name) * 2),52, current_level.name, COLOR_Black, 0x00);
#else
	char line_14[] = "              ";
	char line_20[] = "                    ";

	GrayUserDrawStr(0,32,    line_14, COLOR_LGray, 0xf0);
	GrayUserDrawStr(4*14+1,32, " NEXT LEVEL ", COLOR_White, 0x00);
	GrayUserDrawStr(4*26,32, line_14, COLOR_LGray, 0xf0);

	SetUserFont(apogee_font8x10 - 32 * 10, 8, 10);
	GrayUserDrawStr(0,44, line_20, COLOR_DGray, 0xff);
	GrayUserDrawStr(0,61, line_20, COLOR_DGray, 0xff);
	GrayUserDrawStr(0,52, line_20, COLOR_White, 0xff);
	GrayUserDrawStr(80 - (strlen(current_level.name) * 4),52, current_level.name, COLOR_Black, 0x00);
	SetUserFont(font4x7 - 32 * 7, 4, 7);
#endif
}


void InitLevel(void)
{
	memcpy(&current_level, external_level + (level_number-1), sizeof(LEVEL_Type));

	bonus_points_level = current_level.bonus_points;

	ConstructPlatformLevel();

	level_toggle_bits = current_level.toggle_bits;
}


uint4 FindMagicMarker(uint4 start_address)
{
	int2 i;
	uint4* p_magic_marker;

	for (i = 0; i < 256; i += 2)
	{
		p_magic_marker = (uint4*)(start_address + i);

		if (*p_magic_marker == 0x4a4d3839)  // search for 'JM89'
		{
			p_magic_marker++;
			return (uint4)p_magic_marker;
		}
	}

	return 0;
}


void DrawSelectScreen(void)
{
	int2 curChoice = 0;
	int2 maxChoice = 0;
	int2 i = 0;
	int2 j = 0;
	char* file_name;
	char* name;
	//register uint4 file_address;
	uint4 file_address;
	//KeyboardInput keyPress = KBD_NoKey;
	bool done = false;
	bool dummy = false;
	char temp_str[48];
	const char file_type[] = "jml";

	GrayFillScreen(COLOR_Black);
	SwapPlaneBuffers();
	GrayFillScreen(COLOR_Black);
	SwapPlaneBuffers();

	ResetAutoQuit();

	memset(level_pack_list, 0, sizeof(level_pack_list));

	file_name = FindFileType((char*)file_type, false);

	// find all level packs. Max supported at this time is 10
	while (!done)
	{
		if ((file_name == 0) || (i >= 10))
		{
			done = true;
			maxChoice = i;		
		}
		else
		{
			strncpy(level_pack_list[i].file_name, file_name, 8);

			file_address = (uint4)OpenFile(file_name);

			name = (char*)FindMagicMarker((uint4) file_address);

			memset(level_pack_list[i].name, 32, 31);
			level_pack_list[i].name[31] = 0;
			strncpy(level_pack_list[i].name, name, strlen(name));

			level_pack_list[i].compression_address = (uint4)(name) + strlen(name) + 4;

			file_name = FindFileType((char*)file_type, true);
		}

		i++;
	}
	
	if (maxChoice > 0)
		curChoice = 0;
	
	done = false;
	
	while(!done)
	{
		CheckAutoQuit(TITLE_AUTOTIMER);
		CheckExitGame();
		CheckResetGame();
		CheckAdjustContrast();
		
		if (exit_game || reset_game)
		{
			done = true;
		}
		else if (_keytest (RR_UP))
		{
			// wait until key is released
			while (_keytest (RR_UP));
			ResetAutoQuit();
			curChoice--;
		}
		else if (_keytest (RR_DOWN))
		//else if (keyPress == KBD_Down)
		{
			// wait until key is released
			while (_keytest (RR_DOWN));
			ResetAutoQuit();
			curChoice++;
		}
		else if (_keytest (RR_2ND))
		{
			while (_keytest (RR_2ND));

#ifdef USE_INTERNAL_LEVEL_PACK
			external_level = &internal_level_pack[0];
#else
			ttunpack_decompress((uint1*)level_pack_list[curChoice].compression_address, external_level_buffer);
			external_level = (LEVEL_Type*)FindMagicMarker((uint4) external_level_buffer);
#endif

			for (j = 0; j <= maxChoice; j++)
			{
				CloseFile(level_pack_list[j].file_name);
			}

			game_stage++;
			done = true;
		}
		
		if (curChoice >= maxChoice)
			curChoice = 0;	
		else if (curChoice < 0)
			curChoice = maxChoice - 1;

		GrayFillScreen(COLOR_Black);

		GrayUserDrawStr( 0,6, (char*)"****** JUMPMAN LEVEL PACK OPTIONS ******", COLOR_White, 0);

		for (i = 0; i < maxChoice; i++)
		{
			uint2 highlight = 0;

			if (i == curChoice)
			{
				highlight = 0xf0;
			}

			sprintf(temp_str, "%7d. %s", i+1, level_pack_list[i].name);
			GrayUserDrawStr( 0,18 + 8*i, temp_str, COLOR_White, highlight);
		}

		for (i = maxChoice; i < 10; i++)
		{
			sprintf(temp_str, "%7d. %s", i+1, "-- EMPTY --");
			GrayUserDrawStr( 0,18 + 8*i, temp_str, COLOR_DGray, 0);
		}

		SwapPlaneBuffers();
		Sleep(20);
	}
}





